@@ -7,6 +7,7 @@ import os
 import hashlib
 import random
 
+from mdutils.mdutils import MdUtils
 from datetime import datetime
 from PIL import Image
 from statistics import mean
@@ -396,9 +397,10 @@ class DataVisualizationUtilities:
             ax = fig.add_subplot(2, 10, idx+1, xticks=[], yticks=[])
             plt.imshow(self._im_convert(images[idx], mean=train_utils.mean, std=train_utils.std))
             ax.set_title(train_utils.classes[labels[idx].numpy()])
+
                 
     
-    def display_metric_results(self, fold:int, train_utils, figsize=(7, 7), img_dir="./incorrect_images"):
+    def display_metric_results(self, fold:int, train_utils, figsize=(7, 7), img_dir="./incorrect_images", media_dir="./media"):
         """
         Displays classification report and confusion matrix.
 
@@ -412,6 +414,8 @@ class DataVisualizationUtilities:
             Tuple representing the dimensions of the figure in inches, by default (7, 7).
         `img_dir` : `str`, `optional`\n
             String representation of the path to the incorrect images directory, by default "./incorrect_images".
+        `media_dir` : `str`\n
+            String representation of the path to the media directory.
             
         """        
         
@@ -422,7 +426,9 @@ class DataVisualizationUtilities:
         xticks = yticks = train_utils.classes
         
         print("Classification Report\n")
-        print(classification_report(y_true.cpu(), y_pred.argmax(dim=1).cpu(), target_names=xticks))
+        clr = classification_report(y_true.cpu(), y_pred.argmax(dim=1).cpu(), target_names=xticks)
+        print(clr)
+        train_utils.md_file.insert_code(str(clr))
         print("Confusion Matrix")
         cnf_mat = confusion_matrix(y_true.cpu(), y_pred.argmax(dim=1).cpu())
 
@@ -431,8 +437,8 @@ class DataVisualizationUtilities:
         sns.heatmap(cnf_mat, xticklabels=xticks, yticklabels=yticks, annot=True, cmap="Blues_r")
         plt.ylabel('Ground Truth')
         plt.xlabel('Predictions')
-        plt.title("Confusion Matrix " + train_utils.model_name)
-        plt.show()  
+        return plt
+        
         
         
     def display_results(self, loss:float, acc:float, val_loss:float, val_acc:float, title:str, figsize=(7, 7)):
@@ -475,7 +481,7 @@ class DataVisualizationUtilities:
         plt.ylim([0, y_upper_bound+y_upper_bound*0.2])
         plt.title('Training and Validation Loss '+ title)
         plt.xlabel('epoch')
-        plt.show() 
+        return plt
         
         
     def display_benchmark_results(self, pred_times1, pred_times2, model_name1:str, model_name2:str, figsize=(7, 7), shade=True, 
@@ -517,6 +523,7 @@ class DataVisualizationUtilities:
         plt.title(f'Benchmark Results')
         plt.legend(loc="upper right")
         plt.show()
+        # return plt
 
         
     def display_roc_curve(self, fold:int, train_utils, figsize=(7, 7)):
@@ -550,7 +557,8 @@ class DataVisualizationUtilities:
         plt.xlabel("False Positive Rate")
         plt.ylabel("True Positive Rate")
         plt.legend(loc="lower right")
-        plt.show()
+        return plt
+        # plt.show()
 
 
 class TrainingUtilities:
@@ -587,6 +595,7 @@ class TrainingUtilities:
         self.parameters_path = parameters_path
         self.device = device if device else torch.device("cuda" if torch.cuda.is_available() else "cpu")     
         self.model = nn.Module().to(device=self.device)
+        self.md_file = MdUtils(file_name='media/report.md',title='Results')
         
         # VARIABLE INITIALIZATION
         self.model_name = model_name
@@ -936,9 +945,26 @@ class TrainingUtilities:
                     save_image(tensor_img, img_dir+f"/{self.model_name}_fold_{fold}/{hash_}_{labels[idx]}.png")
                                      
         return y_pred, y_true
+            
+            
+    def add_plot_to_md(self, title:str, plot_name:str, plot_path:str):
+        """
+        Adds a plot to the report markdown.
+
+        Parameters
+        ----------
+        `title` : `str`\n
+            The title for the plot
+        `plot_name` : `str`\n
+            The name of the plot
+        `plot_path` : `str`\n
+            String representation of the path to the plot image
+        """        
+        self.md_file.new_line("### {}".format(title))
+        self.md_file.new_paragraph("![{}]({}.png \"{}\")".format(plot_name, plot_path+plot_name, plot_name))
     
     
-    def train(self, model_name:str, model_path:str, inc_path:str, show_graphs=True, dry_run=True, debug=False, max_epoch=1000) -> tuple:
+    def train(self, model_name:str, model_path:str, inc_path:str, media_dir:str, show_graphs=True, dry_run=True, debug=False, max_epoch=1000) -> tuple:
         """
         Wrapper function for the actual training method. Will likely be edited in the future to be more customizable.
 
@@ -950,6 +976,8 @@ class TrainingUtilities:
             String representation of the path to the model checkpoint path.
         `inc_path` : `str`\n
             String representation of the path to the incorrect images path.
+        `media_dir` : `str`\n
+            String representation of the path to the media directory.
         `show_graphs` : `bool`, `optional`\n
             Boolean representing whether or not to display graphs, by default True.
         `dry_run` : `bool`, `optional`\n
@@ -993,7 +1021,13 @@ class TrainingUtilities:
                 
             avg_loss = mean(losses)
             avg_acc = mean(accuracies)
-            print(f'Average Loss: {avg_loss:.5f}  |  Average Accuracy: {avg_acc:.5f}')
+            
+            results = f'Average Loss: {avg_loss:.5f}  |  Average Accuracy: {avg_acc:.5f}'
+            print(results)
+            
+            self.md_file.new_paragraph(results)
+            self.md_file.new_table_of_contents(table_title='Plots', depth=1)
+            self.md_file.create_md_file()
             return avg_loss, avg_acc
             
         else:
@@ -1006,19 +1040,25 @@ class TrainingUtilities:
             optimizer = torch.optim.Adam(self.model.parameters(), lr=self.eta)
             lr_scheduler = torch.optim.lr_scheduler.ReduceLROnPlateau(optimizer, mode='min', factor=self.factor, patience=self.lr_patience, verbose=True)
             loss, acc = self._train(train_dataset, test_dataset, model_path, criterion, optimizer, fold+1, ascii_=True, scheduler=lr_scheduler, dry_run=dry_run, 
-                                    show_graphs=show_graphs, inc_path=inc_path, max_epoch=max_epoch)
+                                    show_graphs=show_graphs, inc_path=inc_path, max_epoch=max_epoch, media_dir=media_dir)
             losses.append(loss)
             accuracies.append(acc)
             
             avg_loss = mean(losses)
             avg_acc = mean(accuracies)
-            print(f'Average Loss: {avg_loss:.5f}  |  Average Accuracy: {avg_acc:.5f}')
+            
+            results = f'Average Loss: {avg_loss:.5f}  |  Average Accuracy: {avg_acc:.5f}'
+            print(results)
+            
+            self.md_file.new_paragraph(results)
+            self.md_file.new_table_of_contents(table_title='Plots', depth=1)
+            self.md_file.create_md_file()
             return avg_loss, avg_acc
     
     
     # https://stackoverflow.com/questions/58996242/cross-validation-for-mnist-dataset-with-pytorch-and-sklearn
     def _train(self, train_dataset:Dataset, test_dataset:Dataset, filepath:str, criterion, optimizer, fold:int, max_epoch=1000, 
-               scheduler=None, shuffle=True, ascii_=False, show_graphs=True, dry_run=False, inc_path="") -> tuple:
+               scheduler=None, shuffle=True, ascii_=False, show_graphs=True, dry_run=False, inc_path="incorrect_images/", media_dir="media/") -> tuple:
         """Does the actual training. Implements early stopping and some debugging.
         """        
         
@@ -1052,17 +1092,42 @@ class TrainingUtilities:
             print(f'Early Stopping Patience at: {es_counter}')
                 
             if es_counter == self.patience:
+                self.model.eval()
+                
+                title = self.model_name
+                
+                # RESULTS GRAPH
+                results = "results_graph_{}_{}".format(self.model_name, fold)
+                results_graph = DataVisualizationUtilities().display_results(train_total_loss, train_total_acc, val_total_loss, val_total_acc, 
+                                                                         title=early_stopping.model_name)
+                results_graph.savefig(media_dir+"/"+results)
+                self.add_plot_to_md("{} training results fold {}".format(self.model_name, fold), results, media_dir)
+                
+                
+                # METRICS GRAPH
+                metrics = "metrics_graph_{}_{}".format(self.model_name, fold)
+                metrics_graph = DataVisualizationUtilities().display_metric_results(fold=fold, train_utils=self, img_dir=inc_path)
+                metrics_graph.savefig(media_dir+"/"+metrics)
+                self.add_plot_to_md("{} classification report fold {}".format(self.model_name, fold), metrics, media_dir)
+                
+                
+                # ROC GRAPH
+                roc = "roc_graph_{}_{}".format(self.model_name, fold)
+                roc_graph = DataVisualizationUtilities().display_roc_curve(0, train_utils=self)
+                roc_graph.savefig(media_dir+"/"+roc)
+                self.add_plot_to_md("{} roc curve fold {}".format(self.model_name, fold), roc, media_dir)
+            
+            
+                # DISPLAY GRAPHS
                 if show_graphs:
-                    self.model.eval()
-                    DataVisualizationUtilities().display_results(train_total_loss, train_total_acc, val_total_loss, val_total_acc, 
-                                                       title=early_stopping.model_name)
-                    
-                    DataVisualizationUtilities().display_metric_results(fold=fold, train_utils=self, img_dir=inc_path)
+                    results_graph.show()
+                    metrics_graph.show()
+                    roc_graph.show()
                     
                 break
             
             epoch += 1
-            
+
         return early_stopping.min_loss, early_stopping.max_acc
     
                 
